"""Classes interfacing input and output parameters in Fluent.
"""

from collections.abc import Mapping, MutableMapping
from typing import Iterator, Union

from ansys.fluent.core.services.scheme_eval import Symbol
from ansys.fluent.core.session import Session

V = Union[int, float, str]


class _InputParametersSettingsImpl(MutableMapping):
    """
    InputParameters implementation using settings API
    """

    def __init__(self, named_expressions, unit_label_getter):
        self._named_expressions = named_expressions
        self._unit_label_getter = unit_label_getter

    def _get_input_parameter_names(self):
        return [
            k
            for k, v in self._named_expressions().items()
            if v.get("input_parameter", True) and not v.get("output_parameter")
        ]

    def __setitem__(self, name: str, value: V) -> None:
        if name not in self._get_input_parameter_names():
            raise LookupError(f"Input parameter {name} doesn't exist.")
        if not (
            isinstance(value, int) or isinstance(value, float) or isinstance(value, str)
        ):
            raise TypeError(f"Value {value} should be of type int, float or string.")
        # TODO: Check if the input str is of form "<float> [<valid unit label>]"
        if isinstance(value, int) or isinstance(value, float):
            unit_label = self._unit_label_getter(name)
            value = f"{value} [{unit_label}]" if unit_label else str(value)
        self._named_expressions[name].definition = value

    def __delitem__(self, name: str) -> None:
        raise NotImplementedError()

    def __getitem__(self, name: V) -> str:
        if name not in self._get_input_parameter_names():
            raise LookupError(f"Parameter {name} doesn't exist.")
        return self._named_expressions[name].definition()

    def __len__(self) -> int:
        return len(self._get_input_parameter_names())

    def __iter__(self) -> Iterator[str]:
        for name in self._get_input_parameter_names():
            yield name


class _InputParametersSchemeImpl(MutableMapping):
    """
    InputParameters implementation using scheme-eval API
    """

    def __init__(self, scheme_eval, unit_label_getter):
        self._scheme_eval = scheme_eval
        self._unit_label_getter = unit_label_getter

    def _get_parameter_names(self):
        return self._scheme_eval(
            "(send (get expressions-package named-expression-manager) get-names)"
        )

    def __setitem__(self, name: str, value: V) -> None:
        if name not in self._get_parameter_names():
            raise LookupError(f"Input parameter {name} doesn't exist.")
        if not (
            isinstance(value, int) or isinstance(value, float) or isinstance(value, str)
        ):
            raise TypeError(f"Value {value} should be of type int, float or string.")
        # TODO: Check if the input str is of form "<float> [<valid unit label>]"
        if isinstance(value, int) or isinstance(value, float):
            unit_label = self._unit_label_getter(name)
            value = f"{value} [{unit_label}]" if unit_label else str(value)
        self._scheme_eval(
            f'(send (send (get expressions-package named-expression-manager) get-object-by-name "{name}") set-var! \'definition "{value}")'  # noqa: E501
        )

    def __delitem__(self, name: str) -> None:
        raise NotImplementedError()

    def __getitem__(self, name: str) -> V:
        if name not in self._get_parameter_names():
            raise LookupError(f"Input parameter {name} doesn't exist.")
        return self._scheme_eval(
            f'(send (send (get expressions-package named-expression-manager) get-object-by-name "{name}") get-var \'definition)'  # noqa: E501
        )

    def __len__(self) -> int:
        return len(self._get_parameter_names())

    def __iter__(self) -> Iterator[str]:
        for name in self._get_parameter_names():
            yield name


class InputParameters(MutableMapping):
    """Provides for accessing and modifying input parameter values in Fluent.

    Parameters
    ----------
    session : Session
        Connected Fluent session.

    Examples
    --------
    >>> inp = InputParameters(session)
    >>> print([(k, v) for k, v in inp.items()])
    [('parameter-1', '0.3 [m/s]'), ('parameter-2', '1.2 [m/s]')]
    >>> inp['parameter-1']
    '0.3 [m/s]'
    >>> inp['parameter-1'] = '0.5 [m/s]'
    >>> inp['parameter-1']
    '0.5 [m/s]'
    >>> inp.get_unit_label('parameter-1')
    'm/s'
    >>> inp['parameter-1'] = '40 [cm/s]'
    >>> inp['parameter-1']
    '40 [cm/s]'
    >>> inp.get_unit_label('parameter-1')
    'cm/s'
    >>> inp['parameter-1'] = 50
    >>> inp['parameter-1']
    '50 [cm/s]'

    """

    def __init__(self, session: Session):
        """Initialize input parameters.

        Parameters
        ----------
        session : Session
        Connected Fluent session.
        """
        self._session = session
        try:
            self._impl = _InputParametersSettingsImpl(
                self._session.setup.named_expressions, self.get_unit_label
            )
        except AttributeError:
            self._impl = _InputParametersSchemeImpl(
                self._session.scheme_eval.scheme_eval, self.get_unit_label
            )

    def get_unit_label(self, name: str) -> str:
        """Get the unit label of a Fluent input parameter.

        Parameters
        ----------
        name : str
            Name of the Fluent input parameter.

        Returns
        -------
        str
            Unit label of the Fluent input parameter.

        """
        value = self[name]
        value_split = value.split(maxsplit=1)
        if len(value_split) == 1:
            return ""
        else:
            return value_split[1].lstrip("[").rstrip("]")

    def __setitem__(self, name: str, value: V) -> None:
        """Set value of an input parameter.

        Parameters
        ----------
        name : str
            Name of the input parameter
        value: Union[int, float, str]
            Value in either integer, float or string format.
            Integer or float values are set in Fluent with unit-label returned by
            ``get_unit_label`` method.
            Strings are accepted in the form "<value> [<unit-label>]", e.g., "5 [m/s]".
            For Fluent 22.2, The unit-label should be same as unit-label returned by
            ``get_unit_label`` method
            For Fluent 23,1, the unit-label should be a valid unit-label in Fluent.

        """
        self._impl[name] = value

    def __delitem__(self, name: str) -> None:
        """Deletion is not supported."""
        raise RuntimeError(f"Cannot delete parameter {name}")

    def __getitem__(self, name: str) -> V:
        """Get value of an input parameter.

        Parameters
        ----------
        name : str
            Name of the input parameter

        Returns
        -------
        Union[int, float, str]
            Value of the input parameter
            If the parameter has units, a string in the form
            "<value> [<unit-label>]", e.g., "5 [m/s]" is returned.
            If the parameter doesn't have units, a real number is returned.

        """
        value = self._impl[name]
        try:
            return float(value)
        except ValueError:
            return value

    def __len__(self) -> int:
        """Get the number of input parameters.

        Returns
        -------
        int
            The number of input parameters.

        """
        return len(self._impl)

    def __iter__(self) -> Iterator[str]:
        """Get an iterator over input parameters.

        Returns
        -------
        Iterator[str]
            Iterator over input parameters

        """
        for name in self._impl:
            yield name


class OutputParameters(Mapping):
    """Provides for accessing output parameter values in Fluent.

    Parameters
    ----------
    session : Session
        Connected Fluent session.

    Examples
    --------
    >>> outp = OutputParameters(session)
    >>> print([(k, v) for k, v in outp.items()])
    [('report-def-1', '10 [Pa]'), ('report-def-2', '20 [Pa]')]
    >>> outp['report-def-1']
    '10 [Pa]'

    """

    def __init__(self, session: Session):
        """Initialize InputParameters.

        Parameters
        ----------
        session : Session
        Connected Fluent session.
        """
        self._session = session

    def _get_parameter_names(self):
        return self._session.scheme_eval.scheme_eval(
            "(map (lambda (p) (send p get-name)) (get-all-output-parameters))"
        )

    def _get_unit_label(self, name: str) -> str:
        if name not in self._get_parameter_names():
            raise LookupError(f"Output parameter {name} doesn't exist.")
        unit_label = ""
        units = self._session.scheme_eval.scheme_eval(
            f'(send (get-output-parameter "{name}") get-units)'
        )
        if units:
            unit = units[0]
            if isinstance(unit, Symbol):
                unit = unit.str
            if isinstance(unit, str):
                unit_label = self._session.scheme_eval.scheme_eval(
                    f'(units/inquire-unit-label-string-for-quantity "{unit}")'
                )
        return unit_label

    def __getitem__(self, name: str) -> V:
        """Get the value of an output parameter.

        Parameters
        ----------
        name : str
            Name of the output parameter

        Returns
        -------
        Union[int, float, str]
            Value of the output parameter.
            If the parameter has units, a string in the form
            "<value> [<unit-label>]". For example ``"5 [m/s]"``.
            If the parameter does not have units, a real number is returned.

        """
        if name not in self._get_parameter_names():
            raise LookupError(f"Parameter {name} doesn't exist.")
        value = self._session.scheme_eval.scheme_eval(
            f'(get-output-parameter-value "{name}")'
        )
        unit_label = self._get_unit_label(name)
        return f"{value} [{unit_label}]" if unit_label else value

    def __len__(self) -> int:
        """Get the number of output parameters.

        Returns
        -------
        int
            Number of output parameters.

        """
        return len(self._get_parameter_names())

    def __iter__(self) -> Iterator[str]:
        """Get an iterator over output parameters.

        Returns
        -------
        Iterator[str]
            Iterator over output parameters.

        """
        for name in self._get_parameter_names():
            yield name



"""Classes for running a parametric study in Fluent.

Example
-------
>>> from ansys.fluent.parametric import ParametricStudy

Instantiate the study from a Fluent session that has already read a case:

>>> study1 = ParametricStudy(session.parametric_studies).initialize()

Access and modify the input parameters of the base design point:

>>> ip = study1.design_points["Base DP"].input_parameters
>>> ip['vel_hot'] = 0.2
>>> study1.design_points["Base DP"].input_parameters = ip

Update the current design point:

>>> study1.update_current_design_point()

Access the output parameters of the base design point:

>>> study1.design_points["Base DP"].output_parameters

Create and update more design points, and then delete them:

>>> dp1 = study1.add_design_point()
>>> dp2 = study1.duplicate_design_point(dp1)
>>> study1.update_all_design_points()
>>> study1.delete_design_points([dp1, dp2])

Create, rename, and delete parametric studies:

>>> study2 = study1.duplicate()
>>> study2.rename("abc")
>>> study1.delete()

Project workflow

>>> from ansys.fluent.parametric import ParametricProject
>>> proj = ParametricProject(session.file.parametric_project, session.parametric_studies, "nozzle_para_named.flprj")  # noqa: E501
>>> proj.save()
>>> proj.save_as(project_filepath="nozzle_para_named1.flprj")
>>> proj.export(project_filepath="nozzle_para_named2.flprj")
>>> proj.archive()

Use a parametric session:

>>> from ansys.fluent.parametric import ParametricSession
>>> session1 = ParametricSession(case_filepath="elbow_params_2.cas.h5")
>>> session1.studies['elbow_params_2-Solve'].design_points['Base DP'].input_parameters  # noqa: E501
>>> study2 = session1.new_study()
>>> session2 = ParametricSession(project_filepath="nozzle_para_named.flprj")
"""
from pathlib import Path
import tempfile
from typing import Any, Dict, List, Optional

import ansys.fluent.core as pyfluent
from ansys.fluent.core import LOG

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__.replace(".", "-"))

BASE_DP_NAME = "Base DP"


class DesignPoint:
    """Provides for accessing and modifying design points in a parametric study.

    Parameters
    ----------
    name : str
        Name of the design point.
    dp_settings

    """

    def __init__(self, name: str, dp_settings: Any):
        self.name = name
        self._dp_settings = dp_settings

    @property
    def input_parameters(self) -> Dict[str, float]:
        """Dictionary of input parameter values by name."""
        return self._dp_settings.input_parameters()

    @input_parameters.setter
    def input_parameters(self, value: Dict[str, float]) -> None:
        self._dp_settings.input_parameters = value

    @property
    def output_parameters(self) -> Dict[str, float]:
        """Dictionary of output parameter values by name."""
        return self._dp_settings.output_parameters()

    @property
    def write_data_enabled(self) -> bool:
        """Whether to write data for the design point."""
        return self._dp_settings.write_data()

    @write_data_enabled.setter
    def write_data_enabled(self, value: bool) -> None:
        self._dp_settings.write_data = value

    @property
    def capture_simulation_report_data_enabled(self) -> bool:
        """Whether to capture simulation report data for the design point."""
        return self._dp_settings.capture_simulation_report_data()

    @capture_simulation_report_data_enabled.setter
    def capture_simulation_report_data_enabled(self, value: bool) -> None:
        self._dp_settings.capture_simulation_report_data = value


class ParametricStudy:
    """Provides for managing parametric studies and their respective design points.

    A parametric study is used to parametrize design points in a Fluent solver
    set up. This class provides the ability to run Fluent for a series of
    design points and access or modify input and output parameters.

    Parameters
    ----------
    parametric_studies :

    session : Session, optional
        Connected Fluent session. The default is ``None``.
    name : str, optional
        Name of the parametric study. The default is ``None``.
    design_points : Dict[str, DesignPoint], optional
        Dictionary of design points under the parametric study by name.
        The default is ``None``.
    """

    def __init__(
        self,
        parametric_studies,
        session=None,
        name: Optional[str] = None,
        design_points: Dict[str, DesignPoint] = None,
    ):
        self._parametric_studies = parametric_studies
        self.session = (
            session if session is not None else (_shared_parametric_study_registry())
        )
        self.name = name
        self.design_points = {}
        if design_points is not None:
            self.design_points = design_points
        self.project_filepath = None
        self.session.register_study(self)

    def get_all_studies(self) -> Dict[str, "ParametricStudy"]:
        """Get all currently active studies.

        Returns
        -------
        Dict[str, "ParametricStudy"]
            Dictionary of all currently active studies.
        """
        return {v.name: v for _, v in self.session._all_studies.items()}

    def initialize(self) -> "ParametricStudy":
        """Initialize the parametric study."""
        if self._parametric_studies.initialize.is_active():
            self.project_filepath = Path(
                tempfile.mkdtemp(
                    prefix="project-",
                    suffix=".cffdb",
                    dir=str(Path.cwd()),  # TODO: should be cwd of server
                )
            )
            self.project_filepath.rmdir()
            old_study_names = self._parametric_studies.get_object_names()
            self._parametric_studies.initialize(
                project_filename=self.project_filepath.stem
            )
            new_study_names = self._parametric_studies.get_object_names()
            self.name = set(new_study_names).difference(set(old_study_names)).pop()
            base_design_point = DesignPoint(
                BASE_DP_NAME,
                self._parametric_studies[self.name].design_points[BASE_DP_NAME],
            )
            self.design_points = {BASE_DP_NAME: base_design_point}
            self.session.current_study_name = self.name
            return self
        else:
            LOG.error("Initialize is not available.")

    def rename(self, new_name: str) -> None:
        """Rename the parametric study.

        Parameters
        ----------
        new_name : str
            New name.
        """
        self._parametric_studies.rename(new_name, self.name)
        self.name = new_name
        self.design_points = {
            k: DesignPoint(k, self._parametric_studies[self.name].design_points[k])
            for k, _ in self.design_points.items()
        }

    @property
    def is_current(self) -> bool:
        """Whether the parametric study is the current parametric study."""
        return self.session.current_study_name == self.name

    def set_as_current(self) -> None:
        """Set the parametric study as the current parametric study."""
        if not self.is_current:
            self._parametric_studies.set_as_current(self.name)
            self.session.current_study_name = self.name

    def duplicate(self, copy_design_points: bool = True) -> "ParametricStudy":
        """Duplicate the current study.

        Parameters
        ----------
        copy_design_points : bool, optional
            Whether to copy the design points from the current study. The
            default is ``True``.

        Returns
        -------
        ParametricStudy
            New instance of the parametric study.
        """
        old_study_names = self._parametric_studies.get_object_names()
        self._parametric_studies.duplicate(copy_design_points=copy_design_points)
        new_study_names = self._parametric_studies.get_object_names()
        clone_name = set(new_study_names).difference(set(old_study_names)).pop()
        current_study = self.get_all_studies()[self.session.current_study_name]
        if copy_design_points:
            clone_design_points = {
                k: DesignPoint(k, self._parametric_studies[clone_name].design_points[k])
                for k, _ in current_study.design_points.items()
            }
        else:
            base_design_point = DesignPoint(
                BASE_DP_NAME,
                self._parametric_studies[clone_name].design_points[BASE_DP_NAME],
            )
            clone_design_points = {BASE_DP_NAME: base_design_point}
        clone = ParametricStudy(
            self._parametric_studies, self.session, clone_name, clone_design_points
        )
        self.session.current_study_name = clone.name
        return clone

    def delete(self) -> None:
        """Delete the parametric study."""
        if self.is_current:
            LOG.error("Cannot delete the current study %s", self.name)
        else:
            del self._parametric_studies[self.name]
            self.session._all_studies.pop(id(self))
            del self

    def use_base_data(self) -> None:
        """Use base data for the parametric study."""
        self._parametric_studies.use_base_data()

    def import_design_table(self, filepath: str) -> None:
        """Import the design table for the parametric study.

        Parameters
        ----------
        filepath : str
            Filepath for the design table.
        """
        self._parametric_studies.import_design_table(filepath=filepath)

    def export_design_table(self, filepath: str) -> None:
        """Export the design table for the parametric study.

        Parameters
        ----------
        filepath : str
            Filepath to export the design table to.
        """
        self._parametric_studies.export_design_table(filepath=filepath)

    @property
    def current_design_point(self) -> DesignPoint:
        """Get the current design point.

        This is the current design point within the design points under the
        parametric study.
        """
        dp_name = self._parametric_studies[self.name].current_design_point()
        return self.design_points[dp_name]

    def add_design_point(
        self,
        write_data: bool = False,
        capture_simulation_report_data: bool = True,
    ) -> DesignPoint:
        """Add a new design point under the parametric study.

        Parameters
        ----------
        write_data : bool, optional
            Whether to write data for the design point. The default
            is ``False``.
        capture_simulation_report_data : bool, optional
            Whether to capture simulation report data for the design
            point. The default is ``True``.

        Returns
        -------
        DesignPoint
            New design point.
        """
        self.set_as_current()
        dp_settings = self._parametric_studies[self.name].design_points
        dps_before = dp_settings.get_object_names()
        dp_settings.create_1(
            write_data=write_data,
            capture_simulation_report_data=capture_simulation_report_data,
        )
        dps_after = dp_settings.get_object_names()
        dp_name = set(dps_after).difference(set(dps_before)).pop()
        design_point = DesignPoint(
            dp_name,
            self._parametric_studies[self.name].design_points[dp_name],
        )
        self.design_points[dp_name] = design_point
        return design_point

    def delete_design_points(self, design_points: List[DesignPoint]) -> None:
        """Delete a list of design points.

        Parameters
        ----------
        design_points : List[DesignPoint]
            List of design points to delete.
        """
        if self.current_design_point in design_points:
            LOG.error(
                "Cannot delete the current design point %s",
                self.current_design_point.name,
            )
            design_points.remove(self.current_design_point)
        dp_settings = self._parametric_studies[self.name].design_points
        dp_settings.delete_design_points(
            design_points=[dp.name for dp in design_points]
        )
        for design_point in design_points:
            self.design_points.pop(design_point.name)
            del design_point

    def duplicate_design_point(self, design_point: DesignPoint) -> DesignPoint:
        """Duplicate the design point.

        Parameters
        ----------
        design_point : DesignPoint
            Design point.

        Returns
        -------
        DesignPoint
            New design point.
        """
        dp_settings = self._parametric_studies[self.name].design_points
        dps_before = dp_settings.get_object_names()
        dp_settings.duplicate(design_point=design_point.name)
        dps_after = dp_settings.get_object_names()
        new_dp_name = set(dps_after).difference(set(dps_before)).pop()
        new_dp = DesignPoint(
            new_dp_name,
            self._parametric_studies[self.name].design_points[new_dp_name],
        )
        self.design_points[new_dp_name] = new_dp
        return new_dp

    def save_journals(self, separate_journals: bool) -> None:
        """Save journals.

        Parameters
        ----------
        separate_journals : bool
            Whether to save a separate journal for each design point.
        """
        dp_settings = self._parametric_studies[self.name].design_points
        dp_settings.save_journals(separate_journals=separate_journals)

    def clear_generated_data(self, design_points: List[DesignPoint]) -> None:
        """Clear the generated data for a list of design points.

        Parameters
        ----------
        design_points : List[DesignPoint]
            List of design points.
        """
        dp_settings = self._parametric_studies[self.name].design_points
        dp_settings.clear_generated_data(
            design_points=[dp.name for dp in design_points]
        )

    def load_current_design_point_case_data(self) -> None:
        """Load case data of the current design point."""
        dp_settings = self._parametric_studies[self.name].design_points
        dp_settings.load_case_data()

    def update_current_design_point(self) -> None:
        """Update the current design point."""
        dp_settings = self._parametric_studies[self.name].design_points
        dp_settings.update_current()

    def update_all_design_points(self) -> None:
        """Update all design points."""
        dp_settings = self._parametric_studies[self.name].design_points
        dp_settings.update_all()

    def update_selected_design_points(self, design_points: List[DesignPoint]) -> None:
        """Update a list of design points.

        Parameters
        ----------
        design_points : List[str]
            List of design points.
        """
        dp_settings = self._parametric_studies[self.name].design_points
        dp_settings.update_selected(design_points=[dp.name for dp in design_points])


class ParametricProject:
    """Provides the parametric project workflow.

    Attributes
    ----------
    project_filepath : str
        Filepath of the project.

    Methods
    -------
    open(project_filepath, load_case)
        Open a project.
    save()
        Save a project.
    save_as(project_filepath)
        Save a project as another project.
    export(project_filepath, convert_to_managed)
        Save a project as a copy.
    archive(archive_name)
        Archive a project.
    """

    def __init__(
        self,
        parametric_project,
        parametric_studies,
        project_filepath: str,
        session=None,
        open_project: bool = True,
    ):
        self._parametric_project = parametric_project
        self._parametric_studies = parametric_studies
        self.project_filepath = project_filepath
        self.session = (
            session if session is not None else (_shared_parametric_study_registry())
        )
        if open_project:
            self.open(project_filepath=project_filepath)

    def open(
        self, project_filepath: str = "default.flprj", load_case: bool = True
    ) -> None:
        """Open a project.

        Parameters
        ----------
        project_filepath : str, optional
            Project filename. The default is ``"default.flprj"``.
        load_case : bool, optional
            Whether to load the current case. The default ``True``.
        """
        self._parametric_project.open(
            project_filename=str(Path(project_filepath).resolve()),
            load_case=load_case,
        )
        self.project_filepath = project_filepath
        for study_name in self._parametric_studies.get_object_names():
            study = ParametricStudy(self._parametric_studies, self.session, study_name)
            dps_settings = self._parametric_studies[study_name].design_points
            for dp_name in dps_settings.get_object_names():
                study.design_points[dp_name] = DesignPoint(
                    dp_name, dps_settings[dp_name]
                )

    def save(self) -> None:
        """Save the project."""
        self._parametric_project.save()

    def save_as(self, project_filepath: str) -> None:
        """Save the project as another project.

        Parameters
        ----------
        project_filepath : str
            Filepath to save the new project to.
        """
        self._parametric_project.save_as(project_filename=project_filepath)

    def export(self, project_filepath: str, convert_to_managed: bool = False) -> None:
        """Save the project as a copy.

        Parameters
        ----------
        project_filepath : str
            Name for the new project.
        convert_to_managed : bool
            Whether to convert the project to a managed project.
        """
        self._parametric_project.save_as_copy(
            project_filename=project_filepath,
            convert_to_managed=convert_to_managed,
        )

    def archive(self, archive_path: str = None) -> None:
        """Archive the project.

        Parameters
        ----------
        archive_name : str, optional
            Mame of the archive file.
        """
        if not archive_path:
            archive_path = str(Path(self.project_filepath).with_suffix(".flprz"))
        self._parametric_project.archive(archive_name=archive_path)


class ParametricSessionLauncher:
    """Provides for launcheing Fluent for parametric sessions.

    Methods
    -------
    __call__(*args, **kwargs)
        Launch a Fluent session.
    """

    def __init__(self, *args, **kwargs):
        self._args = args
        self._kwargs = kwargs

    def __call__(self):
        self._kwargs["mode"] = "solver"
        return pyfluent.launch_fluent(*self._args, **self._kwargs)


class ParametricStudyRegistry:
    def __init__(self):
        self._all_studies: Dict[int, "ParametricStudy"] = {}
        self.current_study_name = None

    def register_study(self, study):
        self._all_studies[id(study)] = study


class ParametricSession(ParametricStudyRegistry):
    """Provides for encapsulating studies and projects.

    Attributes
    ----------
    studies : Dict[str, ParametricStudy]
        Dictionary of parametric studies by their names within the session.
    project : ParametricProject
        Name of the parametric project if a project file is to be read.

    Methods
    -------
    new_study()
        Create a study.
    delete_study(self, study_name)
        Delete a study.
    rename_study(self, new_name, old_name)
        Rename a study.
    start_transcript()
        Start streaming of a Fluent transcript.
    stop_transcript()
        Stop streaming of a Fluent transcript.
    """

    def __init__(
        self,
        case_filepath: str = None,
        project_filepath: str = None,
        launcher: Any = ParametricSessionLauncher(),
        start_transcript: bool = False,
    ):
        """Instantiate a ParametricSession.

        Parameters
        ----------
        case_filepath : str, optional
            Case file name. The default is ``None``.
        project_filepath : str, optional
            Project file name. The default is ``None``.
        launcher : _type_, optional
            Fluent launcher. The default is ``ParametricSessionLauncher()``.
        start_transcript : bool, optional
            Whether to start streaming of a Fluent transcript. The default
            is ``False``.
        """
        super().__init__()
        self.studies = {}
        self.project = None
        self._session = launcher()
        self.scheme_eval = self._session.scheme_eval.scheme_eval
        self.scheme_eval(
            "(set parametric-study-dependents-manager " "save-project-at-exit? #f)"
        )
        if not start_transcript:
            self.stop_transcript()
        if case_filepath is not None:
            self._session.file.read(file_name=case_filepath, file_type="case")
            study = ParametricStudy(self._session.parametric_studies, self).initialize()
            self.studies[study.name] = study
            self.project = ParametricProject(
                parametric_project=self._session.file.parametric_project,
                parametric_studies=self._session.parametric_studies,
                project_filepath=str(study.project_filepath),
                open_project=False,
                session=self,
            )
        elif project_filepath is not None:
            self.project = ParametricProject(
                parametric_project=self._session.file.parametric_project,
                parametric_studies=self._session.parametric_studies,
                project_filepath=project_filepath,
                session=self,
            )
            studies_settings = self._session.parametric_studies
            for study_name in studies_settings.get_object_names():
                study = ParametricStudy(studies_settings, self, study_name)
                dps_settings = studies_settings[study_name].design_points
                for dp_name in dps_settings.get_object_names():
                    study.design_points[dp_name] = DesignPoint(
                        dp_name, dps_settings[dp_name]
                    )
                self.studies[study_name] = study
            self.current_study_name = self._session.current_parametric_study()

    def new_study(self) -> ParametricStudy:
        """Create a new study.

        Returns
        -------
        ParametricStudy
            New study.
        """
        study = self.studies[self.current_study_name].duplicate()
        self.studies[study.name] = study
        return study

    def delete_study(self, study_name: str) -> None:
        """Delete a study.

        Parameters
        ----------
        study_name : str
            Study name.
        """
        study = self.studies[study_name]
        if study.is_current:
            LOG.error("Cannot delete the current study %s", study_name)
        else:
            study.delete()
            self.studies.pop(study_name)

    def rename_study(self, new_name: str, old_name: str) -> None:
        """Rename a study.

        Parameters
        ----------
        new_name : str
            New name.
        old_name : str
            Current name.
        """
        study = self.studies.pop(old_name)
        study.rename(new_name)
        self.studies[new_name] = study

    def exit(self) -> None:
        self._session.exit()

    def __enter__(self):
        return self

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any):
        self._session.exit()

    def start_transcript(self) -> None:
        """Start streaming of a Fluent transcript."""
        self._session.start_transcript()

    def stop_transcript(self) -> None:
        """Stop streaming of a Fluent transcript."""
        self._session.stop_transcript()


def _shared_parametric_study_registry():
    if _shared_parametric_study_registry.instance is None:
        _shared_parametric_study_registry.instance = ParametricStudyRegistry()
    return _shared_parametric_study_registry.instance


_shared_parametric_study_registry.instance = None


from ansys.fluent.parametric.parameters import InputParameters, OutputParameters



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ansys', 'ansys.fluent.parametric', 'ansys.fluent.parametric.local']

package_data = \
{'': ['*']}

install_requires = \
['ansys-fluent-core>=0.12.dev0,<1.0', 'h5py>=3.7.0']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>=4.0,<5.0']}

setup_kwargs = {
    'name': 'ansys-fluent-parametric',
    'version': '0.6.dev0',
    'description': 'A python wrapper for Ansys Fluent parametric workflows',
    'long_description': 'PyFluent-Parametric\n===================\n|pyansys| |pypi| |GH-CI| |MIT| |black|\n\n.. |pyansys| image:: https://img.shields.io/badge/Py-Ansys-ffc107.svg?logo=data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAACQkWg2AAABDklEQVQ4jWNgoDfg5mD8vE7q/3bpVyskbW0sMRUwofHD7Dh5OBkZGBgW7/3W2tZpa2tLQEOyOzeEsfumlK2tbVpaGj4N6jIs1lpsDAwMJ278sveMY2BgCA0NFRISwqkhyQ1q/Nyd3zg4OBgYGNjZ2ePi4rB5loGBhZnhxTLJ/9ulv26Q4uVk1NXV/f///////69du4Zdg78lx//t0v+3S88rFISInD59GqIH2esIJ8G9O2/XVwhjzpw5EAam1xkkBJn/bJX+v1365hxxuCAfH9+3b9/+////48cPuNehNsS7cDEzMTAwMMzb+Q2u4dOnT2vWrMHu9ZtzxP9vl/69RVpCkBlZ3N7enoDXBwEAAA+YYitOilMVAAAAAElFTkSuQmCC\n   :target: https://docs.pyansys.com/\n   :alt: PyAnsys\n\n.. |pypi| image:: https://img.shields.io/pypi/v/ansys-fluent-parametric.svg?logo=python&logoColor=white\n   :target: https://pypi.org/project/ansys-fluent-parametric\n   :alt: PyPI\n\n.. |GH-CI| image:: https://github.com/pyansys/pyfluent-parametric/actions/workflows/ci_cd.yml/badge.svg\n   :target: https://github.com/pyansys/pyfluent-parametric/actions/workflows/ci_cd.yml\n   :alt: GH-CI\n\n.. |MIT| image:: https://img.shields.io/badge/License-MIT-yellow.svg\n   :target: https://opensource.org/licenses/MIT\n   :alt: MIT\n\n.. |black| image:: https://img.shields.io/badge/code%20style-black-000000.svg?style=flat\n   :target: https://github.com/psf/black\n   :alt: Black\n\nOverview\n--------\nPyFluent-Parametric provides Pythonic access to Ansys Fluent\'s parametric\nworkflows.\n\nDocumentation and issues\n------------------------\nFor comprehensive information on PyFluent-Parametric, see the latest\nrelease `documentation <https://fluentparametric.docs.pyansys.com>`_.\n\nOn the `PyFluent-Parametric Issues <https://github.com/pyansys/pyfluent-parametric/issues>`_,\nyou can create issues to submit questions, report bugs, and request new features. To reach\nthe PyAnsys support team, email `pyansys.support@ansys.com <pyansys.support@ansys.com>`_.\n\nInstallation\n------------\nThe ``ansys-fluent-parametric`` package currently supports Python 3.7 through Python\n3.10 on Windows and Linux.\n\nInstall the latest release from `PyPI\n<https://pypi.org/project/ansys-fluent-parametric/>`_ with:\n\n.. code:: console\n\n   pip install ansys-fluent-parametric\n\nAlternatively, install the latest from `GitHub\n<https://github.com/pyansys/pyfluent-parametric>`_ with:\n\n.. code:: console\n\n   pip install git+https://github.com/pyansys/pyfluent-parametric.git\n\nIf you plan on doing local *development* of PyFluent with Git, install\nwith:\n\n.. code:: console\n\n   git clone https://github.com/pyansys/pyfluent-parametric.git\n   cd pyfluent-parametric\n   pip install pip -U\n   pip install -e .\n\nDependencies\n------------\nYou must have a locally-installed, licensed copy of Ansys to run Fluent. The\nfirst supported version is 2022 R2.\n\nGetting started\n---------------\n\nBasic usage\n~~~~~~~~~~~\nThe following code assumes that a PyFluent session has already been created and a Fluent case\nwith input parameters has been set up. For a full example, see `Defining Parametric Workflows\n<https://fluentparametric.docs.pyansys.com/users_guide/parametric_workflows.html>`_ in\nthe PyFluent-Parametric documentation.\n\n.. code:: python\n\n   from ansys.fluent.parametric import ParametricStudy\n   study_1 = ParametricStudy(session.solver.root.parametric_studies).initialize()\n   input_parameters_update = study_1.design_points["Base DP"].input_parameters\n   input_parameters_update["inlet1_vel"] = 0.5\n   study_1.design_points["Base DP"].input_parameters = input_parameters_update\n   study_1.update_current_design_point()\n   print(study_1.design_points["Base DP"].output_parameters)\n\nLicense and acknowledgments\n---------------------------\nPyFluent-Parametric is licensed under the MIT license.\n\nPyFluent-Parametric makes no commercial claim over Ansys whatsoever. This library\nextends the functionality of Fluent by adding a Python interface to Fluent without\nchanging the core behavior or license of the original software. The use of the\ninteractive Fluent control of PyFluent-Parametric requires a legally licensed\nlocal copy of Fluent.\n\nFor more information, see the `Ansys Fluent <https://www.ansys.com/products/fluids/ansys-fluent>`\npage on the Ansys website.\n',
    'author': 'ANSYS, Inc.',
    'author_email': 'ansys.support@ansys.com',
    'maintainer': 'PyAnsys developers',
    'maintainer_email': 'pyansys.maintainers@ansys.com',
    'url': 'https://github.com/pyansys/pyfluent-parametric',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
